#!/usr/bin/env node
import inquirer from 'inquirer';
import { exec, spawn } from 'child_process';

// 定义框架类型及其动态获取方式
const frameworkTypes = [
  {
    name: "React",
    getOptions: async () => {
      return [
        {
          name: "Create React App",
          command: "npx create-react-app",
          checkCommand: "npx create-react-app --version",
          installCommand: "npm install -g create-react-app",
          getProjectName: async () => {
            const { projectName } = await inquirer.prompt([
              {
                type: "input",
                name: "projectName",
                message: "请输入项目名称：",
                validate: (input) => input.trim().length > 0 || "项目名称不能为空！",
              },
            ]);
            return projectName;
          },
        },
        {
          name: "Vite",
          command: "npm create vite@latest",
          checkCommand: "npm create vite@latest --version",
          installCommand: "npm install -g vite",
        },
        {
          name: "Next.js",
          command: "npx create-next-app",
          checkCommand: "npx create-next-app --version",
          installCommand: "npm install -g create-next-app",
        },
      ];
    },
  },
  {
    name: "Vue",
    getOptions: async () => {
      return [
        {
          name: "Vite",
          checkCommand: "npm create vite@latest --version",
          command: "npm create vite@latest",
          installCommand: "npm install -g vite",
        },
        {
          name: "Nuxt.js",
          checkCommand: "npx nuxi --version",
          command: "npx nuxi init",
          installCommand: "npm install -g nuxi",
        },
      ];
    },
  },
];

// 检查是否已安装对应的框架工具
const checkIfInstalled = (checkCommand, frameworkName) => {
  return new Promise((resolve) => {
    exec(checkCommand, (err, stdout, stderr) => {
      if (err) {
        if (stderr.includes("command not found") || stderr.includes("not found")) {
          console.log(`${frameworkName} 未安装，将自动进行安装。`);
          resolve(false);
        } else {
          console.error(`检测工具时出错：${stderr}`);
          process.exit(1);
        }
      } else {
        const version = stdout.trim();
        console.log(`${frameworkName} 已安装，版本：${version}`);
        resolve(true);
      }
    });
  });
};

// 安装框架工具
const installFramework = (installCommand) => {
  return new Promise((resolve) => {
    console.log(`正在安装必要的工具：${installCommand}`);
    exec(installCommand, (err, stdout, stderr) => {
      if (err) {
        console.error("安装失败：", err);
        resolve(false);
      } else {
        console.log("安装成功！");
        resolve(true);
      }
    });
  });
};

// 根据用户选择创建项目
const createProject = async (framework) => {
  console.log(`正在检查是否已安装 ${framework.name} 的工具...`);
  const isInstalled = await checkIfInstalled(framework.checkCommand, framework.name);

  if (!isInstalled) {
    console.log(`${framework.name} 的工具未安装，正在自动安装...`);
    await installFramework(framework.installCommand);
  }

  // 如果存在 getProjectName 方法，动态获取项目名称
  let projectName = "";
  if (framework.getProjectName) {
    projectName = await framework.getProjectName();
    framework.command += ` ${projectName}`;
  }

  console.log(`正在启动 ${framework.name} 的交互式创建流程...`);
  const child = spawn(framework.command, { shell: true, stdio: 'inherit' });

  child.on('close', (code) => {
    // console.log(`子进程退出，退出码 ${code}`);
  });

  child.on('error', (err) => {
    console.error(`启动子进程时出错：${err.message}`);
  });
};

// 主程序
const main = async () => {
  // 第一层：选择框架类型
  const { type } = await inquirer.prompt([
    {
      type: "list",
      name: "type",
      message: "请选择框架类型：",
      choices: frameworkTypes.map((t) => t.name),
    },
  ]);

  // 动态加载该类型下的具体框架选项
  const frameworkOptions = await frameworkTypes.find(t => t.name === type).getOptions();

  // 第二层：选择具体框架
  const { framework } = await inquirer.prompt([
    {
      type: "list",
      name: "framework",
      message: "请选择具体框架：",
      choices: frameworkOptions.map((f) => f.name),
    },
  ]);

  const selectedFramework = frameworkOptions.find(
    (f) => f.name === framework
  );

  // 创建项目
  await createProject(selectedFramework);
};

// 执行主程序
main();